@@ -19,6 +19,7 @@
     along with INI Library.  If not, see <http://www.gnu.org/licenses/>.
 */
 
+#include "config.h"
 #include <stdlib.h>
 #include <stdio.h>
 #include <string.h>
@@ -467,7 +468,7 @@ int get_test(void)
     double number_double;
     unsigned number_unsigned;
     unsigned long number_ulong;
-    unsigned char logical;
+    unsigned char logical = 0;
     char *str;
     const char *cstr;
     const char *cstrn;
@@ -637,7 +638,6 @@ int get_test(void)
     COLOUT(printf("Convert item to number without strict conversion.\n"));
 
     error = 0;
-    number = 1;
     number = get_int_config_value(item, 0, 10, &error);
     if (error) {
         /* We expected error in this case */
@@ -779,7 +779,7 @@ int get_test(void)
     error = 0;
     logical = get_bool_config_value(item, 1, &error);
     if (!error) {
-        printf("Expect error. Got success.\n");
+        printf("Expect error. Got success. Value %d\n", (int) logical);
         free_ini_config(ini_config);
         return -1;
     }
@@ -946,6 +946,48 @@ int get_test(void)
 
     free_string_config_array(strarray);
 
+    /**********************************************************/
+
+    COLOUT(printf("Get bad string array \n"));
+
+    item = NULL;
+    error = get_config_item("domains", "badarray", ini_config, &item);
+    if(error) {
+        printf("Expected success but got error! %d\n",error);
+        free_ini_config(ini_config);
+        return error;
+    }
+
+    /* Item should be found */
+    if (item == NULL) {
+        printf("Expected success but got NULL.\n");
+        free_ini_config(ini_config);
+        return -1;
+    }
+
+    COLOUT(col_debug_item(item));
+
+    COLOUT(printf("Get bad str array without size.\n"));
+
+    error = 0;
+    strarray = get_string_config_array(item, ",", NULL, &error);
+    if (error) {
+        printf("Expect success got error %d.\n", error);
+        free_ini_config(ini_config);
+        return error;
+    }
+
+    /* Can be used with this cycle */
+    strptr = strarray;
+    while (*strptr != NULL) {
+        COLOUT(printf("[%s]\n",*strptr));
+        strptr++;
+    }
+
+    free_string_config_array(strarray);
+
+    /**********************************************************/
+
     COLOUT(printf("Get long array item\n"));
 
     item = NULL;
@@ -1505,6 +1547,7 @@ int main(int argc, char *argv[])
     char *srcdir = NULL;
     char *rundir = NULL;
     const char inidir[] = "/ini";
+    int len = 0;
 
     if ((argc > 1) && (strcmp(argv[1], "-v") == 0)) verbose = 1;
 
@@ -1512,8 +1555,18 @@ int main(int argc, char *argv[])
 
     srcdir = getenv("srcdir");
     if(srcdir) {
-        rundir = malloc(strlen(srcdir) + sizeof(inidir));
-        sprintf(rundir, "%s%s", srcdir, inidir);
+
+        len = strlen(srcdir) + sizeof(inidir);
+        rundir = malloc(len);
+        if (!rundir) {
+            printf("Failed to allocate memory to store path"
+                   " to the test files %d.\n", ENOMEM);
+            return -1;
+        }
+
+        snprintf(rundir, len, "%s%s", srcdir, inidir);
+
+        errno = 0;
         if(chdir(rundir) != 0) {
             error = errno;
             free(rundir);
