@@ -192,6 +192,7 @@ bool is_absolute_path(const char *path)
 
 int path_concat(char *path, size_t path_size, const char *head, const char *tail)
 {
+    int ret;
     const char *p, *src;
     char *dst, *dst_end;
 
@@ -201,24 +202,59 @@ int path_concat(char *path, size_t path_size, const char *head, const char *tail
     dst_end = path + path_size - 1;             /* -1 allows for NULL terminator */
 
     if (head && *head) {
-        for (p = head; *p; p++);                /* walk to end of head */
-        for (p--; p >= head && *p == '/'; p--); /* skip any trailing slashes in head */
-        if ((p - head) > path_size-1) return ENOBUFS;
-        for (src = head; src <= p && dst < dst_end;) *dst++ = *src++; /* copy head */
+        /* walk to end of head */
+        for (p = head; *p; p++);
+
+        /* skip any trailing slashes in head */
+        for (p--; p > head && *p == '/'; p--);
+
+        /* If the length of head exceeds the buffer size, fail */
+        if ((p - head) > path_size-1) {
+            ret = ENOBUFS;
+            goto fail;
+        }
+
+        /* Copy head into path */
+        for (src = head; src <= p && dst < dst_end;) {
+            *dst++ = *src++;
+        }
     }
     if (tail && *tail) {
-        for (p = tail; *p && *p == '/'; p++);   /* skip any leading slashes in tail */
+        /* skip any leading slashes in tail */
+        for (p = tail; *p && *p == '/'; p++);
+
         if (dst > path)
-            if (dst < dst_end) *dst++ = '/';    /* insert single slash between head & tail */
-        for (src = p; *src && dst <= dst_end;) *dst++ = *src++; /* copy tail */
-        if (*src) return ENOBUFS; /* failed to copy everything */
+            /* insert single slash between head & tail
+             * Making sure not to add an extra if the
+             * preceding character is also a slash
+             * (such as the case where head was the
+             * special-case "/".
+             */
+            if (dst < dst_end && *(dst-1) != '/') {
+                *dst++ = '/';
+            }
+
+        /* Copy the tail into the path */
+        for (src = p; *src && dst < dst_end;) {
+            *dst++ = *src++;
+        }
+
+        /* If we got past dst_end and there is more data
+         * in the src buffer, we should return ENOBUFS
+         */
+        if (*src) {
+            ret = ENOBUFS; /* failed to copy everything */
+            goto fail;
+        }
     }
     *dst = 0;
-    if (dst > dst_end) {
-        return ENOBUFS;
-    }
+
     return SUCCESS;
 
+fail:
+    /* On failure, set the buffer to the empty string for safety */
+    *path = '\0';
+    return ret;
 }
 
 int make_path_absolute(char *absolute_path, size_t absolute_path_size, const char *path)
@@ -321,13 +357,13 @@ char **split_path(const char *path, int *count)
     for (start = end = path; *start; start = end) {
         for (start = end; *start && *start == '/'; start++);
         for (end = start; *end && *end != '/'; end++);
-        if ((component_len = end - start) == 0) break;
+        if ((end - start) == 0) break;
 
         *array_ptr++ = component_ptr;
         while (start < end) *component_ptr++ = *start++;
         *component_ptr++ = 0;
     }
-    *array_ptr++ = NULL;
+    *array_ptr = NULL;
     if (count) *count = n_components;
     return (char **)mem_block;
 }
@@ -529,7 +565,7 @@ int directory_list(const char *path, bool recursive,
     }
 
     for (entry = readdir(dir); entry; entry = readdir(dir)) {
-        prune = false;
+
         if (strcmp(entry->d_name, ".") == 0 ||
             strcmp(entry->d_name, "..") == 0) {
             continue;
@@ -575,25 +611,21 @@ bool is_ancestor_path(const char *ancestor, const char *path)
 {
     char **path_components, **ancestor_components;
     int i, path_count, ancestor_count;
-    bool result;
+    bool result = false;
 
-    result = false;
     path_components = split_path(path, &path_count);
     ancestor_components = split_path(ancestor, &ancestor_count);
 
     if (!path_components || !ancestor_components) {
-        result = false;
         goto exit;
     }
 
     if (ancestor_count >= path_count) {
-        result = false;
         goto exit;
     }
 
     for (i = 0; i < ancestor_count; i++) {
         if (strcmp(path_components[i], ancestor_components[i]) != 0) {
-            result = false;
             goto exit;
         }
     }
